from click import echo, style

class Log:
    def output(self, msg):
        echo(style(" SPKG ", bg="blue") + " " + msg)
    def info(self, msg):
        self.output(style(" INFO ",  bg="magenta") + " " + msg)
    def error(self, msg):
        self.output(style(" ERROR ", bg="red") + " " + msg)
    def warn(self, msg):
        self.output(style(" WARN ",  bg="yellow") + " " + msg)
    def done(self, msg):
        self.output(style(" DONE ",  bg="green") + " " + msg)

if __name__ == "__main__":
    text = "Hello, world!"
    log = Log()
    log.info(text)
    log.error(text)
    log.warn(text)
    log.done(text)


from spkg.hatch import Hatch_cli
from spkg.ci import CI
from spkg.log import Log
from click import style
from subprocess import run

class Action:
    def __init__(self, log: Log, hatch: Hatch_cli, ci: CI):
        self.log = log
        self.ci = ci
        self.hatch = hatch

    def version(self, option):
        if self.ci.is_exist():
            self.log.info("CI detected, only modifying the version number.")
            self.hatch.version(option)
        else:
            self.log.info(f"Releasing {option} update and publishing to pypi.")
            self.hatch.version(option)
            self.build()
            self.release()

    def build(self, clean=True):
        if clean:
            self.hatch.clean()
            self.log.done("Cleaned dist folder.")
        self.hatch.build()
        self.log.done("Built package.")
    
    def create_ci(self):
        name = self.hatch.metadata("name")
        self.log.info(f"Generating publish.yml for {name}.")
        self.ci.create(name, in_project=True)
        self.log.done("Created publish.yml successfully.")

    def release(self):
        self.hatch.publish()
        self.log.done("Published to pypi.")

    def simple_config(self):
        self.log.info("Using simple config.")
        self.hatch.simple_config()
        self.log.done("Configured successfully: no tests, GPL-3.0 license.")
    def restore_config(self):
        self.log.info("Restoring to default config.")
        self.hatch.config_restore()
    def need_option(self, cmd):
        self.log.error(f"Please specify a valid option for {cmd}.")
        self.log.info(f'Use {style(f"spkg {cmd} --help", fg="green")} for more information.')
    def new(self, name, is_cli, create_ci, open_vscode):
        if is_cli:
            self.log.info(f"Creating a new package {name} with Command Line Interface.")
            self.hatch.new_cli(name)
        else:
            self.log.info(f"Creating a new package {name}.")
            self.hatch.new(name)

        if create_ci:
            self.log.info("Creating Publish Workflow for GitHub Actions.")
            self.ci.create(name, in_project=False)

        self.log.done(f"Created package {name} successfully.")
        
        if open_vscode:
            self.log.info("Opening in Visual Studio Code.")
            run(f"code {name}", shell=True)



import sys
import subprocess
import json
from click import style

PYTHON_PATH = sys.executable

class Hatch_cli:
    def hatch(self, args):
        cmd_list = [PYTHON_PATH, "-m", "hatch"] + args
        subprocess.run(cmd_list)
    def hatch_output(self, args):
        cmd_list = [PYTHON_PATH, "-m", "hatch"] + args
        result = subprocess.run(cmd_list, capture_output=True, text=True)
        return result.stdout

    def config(self, key, value):
        self.hatch(["config", "set", key, value])
    def config_restore(self):
        self.hatch(["config", "restore"])
    def simple_config(self):
        self.config_restore()
        self.config("template.licenses.default", "['GPL-3.0-only']")
        self.config("template.plugins.default.tests", "false")
    def metadata(self, field):
        result = self.hatch_output(["project", "metadata", field])
        return result[:-1]
    def clean(self):
        self.hatch(["clean"])
    def build(self):
        self.hatch(["build"])
    def publish(self):
        self.hatch(["publish"])
    def release(self):
        self.hatch(["release"])
    def version(self, version):
        self.hatch(["version", version])
    def new(self, name):
        self.hatch(["new", name])
    def new_cli(self, name):
        self.hatch(["new", "--cli", name])
    def info(self):
        result = self.hatch_output(["project", "metadata"])
        info_dict = json.loads(result)

        deps = info_dict.get('dependencies', ["None"])
        output = f"""{info_dict.get('name')}
Description:
{info_dict.get('description')}
Version: {style(info_dict.get('version'), fg='green')}
License: {style(info_dict.get('license'), fg='blue')}
Dependencies: """
        for dep in deps:
            output += f"{style(dep, fg='magenta')} "
        return output



from os import path, makedirs

WORKFLOW = """\
name: publish

on:
  push:
    branches: [ "main" ]
  pull_request:
    branches: [ "main" ]

jobs:
  build-and-publish:
    runs-on: ubuntu-latest
    environment:
      name: pypi
      url: https://pypi.org/p/{name}
    permissions:
      id-token: write
    steps:
    - uses: actions/checkout@v4
    - name: Set up Python 
      uses: actions/setup-python@v5
      with:
        python-version: '3.8'
    - name: Install dependencies
      run: |
        python -m pip install --upgrade hatch
    - name: Build package
      run: |
        hatch build

    - name: Publish package distributions to PyPI
      uses: pypa/gh-action-pypi-publish@release/v1
"""

CI_DIR = path.join(".github", "workflows")
CI_NAME = "publish.yml"

class CI:
    def is_exist(self):
        return path.exists(path.join(CI_DIR, CI_NAME))
    def create(self, name, in_project):
        cur_ci_dir = CI_DIR
        if not in_project:
            cur_ci_dir = path.join(name, cur_ci_dir)
        makedirs(cur_ci_dir, exist_ok=True)
        ci_file = path.join(cur_ci_dir, CI_NAME)

        with open(ci_file, "w", encoding="utf-8") as f:
            f.write(WORKFLOW.format(name=name))




# SPDX-FileCopyrightText: 2024-present xuanzhi33 <xuanzhi33@qq.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import click

from spkg.__about__ import __version__
from spkg.hatch import Hatch_cli
from spkg.ci import CI
from spkg.log import Log
from spkg.action import Action

log = Log()
hatch = Hatch_cli()
ci = CI()
action = Action(log=log, hatch=hatch, ci=ci)


EPILOG = f"{click.style(' SPKG ', bg='blue')} by xuanzhi33 {click.style('v' + __version__, fg='green')}"


@click.group(context_settings={"help_option_names": ["-h", "--help"]},
             epilog=EPILOG)
@click.version_option(version=__version__, prog_name="spkg")
def spkg():
    """
    SPKG (Simple Packager) - A command line tools for building and uploading python packages.
    """
    pass

@spkg.command()
@click.option("-s", "--simple", is_flag=True, help="Using simple config: no tests, GPL-3.0 license.")
@click.option("-r", "--restore", is_flag=True, help="Restore to default config.")
def config(simple, restore):
    """
    Configure spkg.
    """
    if simple:
        action.simple_config()
    elif restore:
        action.restore_config()
    else:
        action.need_option("config")

@spkg.command()
def patch():
    """
    Release patch update (1.0.1 -> 1.0.2) and publish to pypi.

    If GitHub Actions (publish.yml) is enabled, it will only modify the version number.
    """
    action.version("patch")

@spkg.command()
def minor():
    """
    Release minor update (1.0.1 -> 1.1.0) and publish to pypi.

    If GitHub Actions (publish.yml) is enabled, it will only modify the version number.
    """
    action.version("minor")

@spkg.command()
def major():
    """
    Release major update (1.0.1 -> 2.0.0) and publish to pypi.

    If GitHub Actions (publish.yml) is enabled, it will only modify the version number.
    """
    action.version("major")

@spkg.command()
@click.option("--clean/--no-clean", " /-c", default=True, help="Clean dist folder before building.")
def build(clean):
    """
    Build package.
    """
    action.build(clean)

@spkg.command()
def publish():
    """
    Publish to pypi.
    """
    action.release()

@spkg.command()
@click.option("--create", "-c", is_flag=True, help="Create GitHub Actions Workflow for this package.")
def ci(create):
    """
    Manage Publish GitHub Actions Workflow for this package.
    """
    if create:
        action.create_ci()
    else:
        action.need_option("ci")


@spkg.command()
@click.argument("name")
@click.option("--cli/--no-cli", "-c", default=False, help="Create package with Command Line Interface.")
@click.option("--ci/--no-ci", "-a", default=False, help="Create Publish Workflow for GitHub Actions.")
@click.option("--open/--no-open", "-o", default=False, help="Open in Visual Studio Code when finished.")
def new(name, cli, ci, open):
    """
    Create a new package.
    """

    action.new(name, is_cli=cli, create_ci=ci, open_vscode=open)

@spkg.command()
def info():
    """
    Show package information.
    """
    log.info(hatch.info())


if __name__ == "__main__":
    spkg()